import java.sql.*;
import java.util.Properties;
import java.io.*;
import java.net.*;

public class CommentsServer extends Thread {
  public static final int DEFAULT_PORT = 7777;
  protected int port;
  protected ServerSocket server;
  private static final String driverPrefixURL = "jdbc:odbc:";
  private static String username = null;
  private static String password = null;
  private static String dataSource = null;
  public static String getUsername() {
    return username;
  }
  public static String getPassword() {
    return password;
  }
  public static String getDriverURL() {
    return driverPrefixURL+dataSource;
  }
  public static void main (String args[]) {
    int port=0;
    if (args.length == 1) {
      try {
        port = Integer.parseInt (args[0]);
      } catch (NumberFormatException e) { }
    }
    try {
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    }
    catch (Exception e) {
      System.err.println("Failed to load JDBC/ODBC driver.");
      System.exit (1);
    }
    try {
      // Look for resource file 'odbc.datasource'
      InputStream is = ClassLoader.getSystemResourceAsStream ("odbc.datasource");
      Properties p = new Properties();
      p.load (is);
      dataSource = p.getProperty("datasource.name");
      if (dataSource == null)
        throw new Exception ();
      username = p.getProperty("datasource.username", "");
      password = p.getProperty("datasource.password", "");
    } catch (Exception e) {
      System.err.println("Unable to read resource to get data source");
      System.exit (1);
    }
    new CommentsServer (port);
  }
  public CommentsServer (int port) {
    super ("Comments Server");
    if (port == 0)
      port = DEFAULT_PORT;
    this.port = port;
    try {
      server = new ServerSocket (port);
    } catch (IOException e) {
      System.err.println ("Error creating server");
      System.exit (1);
    }
    start();
  }
  public void run() {
    System.out.println ("Server Running");
    ThreadGroup connections = new ThreadGroup ("Comment Connections");
    connections.setMaxPriority(this.getPriority()-1);
    try {
      while (true) {
        Socket client = server.accept();
        System.out.println ("Connection from: " + client.getInetAddress().getHostName());
        CommentsConnection c = new CommentsConnection (connections, client);
      }
    } catch (IOException e) {
      System.err.println ("Exception listening");
      System.exit (1);
    }
    System.exit (0);
  }
}

class CommentsConnection extends Thread {
  static int counter = 0;
  protected ObjectInputStream in;
  protected PrintWriter out;
  public CommentsConnection (ThreadGroup group, Socket client) {
    super (group, "Connection " + counter++);
    try {
      in = new ObjectInputStream (client.getInputStream ());
      out = new PrintWriter (client.getOutputStream(), true);
    } catch (IOException e) {
      try {
        client.close();
      } catch (IOException e2) { }
      System.err.println ("Unable to connect.");
      return;
    }
    start();
  }
  public void run () {
    try {
      String mode = (String)in.readObject();
      if (mode.equals ("insert")) {
        String name = (String)in.readObject();
        String username = (String)in.readObject();
        String comments = (String)in.readObject();
        try {
          Connection con = DriverManager.getConnection(
              CommentsServer.getDriverURL(),
              CommentsServer.getUsername(), 
              CommentsServer.getPassword());
          PreparedStatement prep = con.prepareStatement("INSERT into comments values (?, ?, ?)");
          prep.setString (1, name);
          prep.setString (2, username);
          prep.setString (3, comments);
          if (prep.executeUpdate () != 1)
            throw new Exception ("Bad Update");
        } catch (Exception e) {
          out.println ("Error updating." + e);
          return;
        }
      } else if (mode.equals ("query")) {
        try {
          Connection con = DriverManager.getConnection(
              CommentsServer.getDriverURL(),
              CommentsServer.getUsername(), 
              CommentsServer.getPassword());
          Statement statement = con.createStatement();
          ResultSet result = statement.executeQuery("SELECT * FROM comments");
          out.println("Name\tEmail\tComments");
          int nameCol = result.findColumn ("name");
          int userCol = result.findColumn ("username");
          int commentsCol = result.findColumn ("comments");
          String name, user, comments;
          while(result.next()) {
            name = result.getString(nameCol);
            user = result.getString(userCol);
            comments = result.getString(commentsCol);
            out.println (name + "\t" + user + "\t" + comments);
          }
          statement.close();
          con.close();
        } catch (Exception e) {
          out.println ("Error querying." + e);
          return;
        }
      } else {
        out.println ("Invalid Command: " + mode);
      }
    } catch (Exception e) {
      out.println ("Error reading Stream." + e);
    }
    out.close();
  }
}
