import java.sql.*;
import java.util.Properties;
import java.io.InputStream;

public class ErrorChecker {
  public static void main (String args[]) {
    String driverPrefixURL = "jdbc:odbc:";
    String username = null;
    String password = null;
    String dataSource = null;
    try {  // Invalid driver
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriverxx");
    } catch (ClassNotFoundException e) {
      System.out.println("If an invalid driver name is provided or the " +
          "class cannot be found, the ClassNotFoundException is thrown.");
    }
    try { // Valid driver
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch (ClassNotFoundException e) {
      System.out.println("Failed to load JDBC/ODBC driver.");
    }
    try { // Resource loading
      // Look for resource file 'odbc.datasource'
      InputStream is = ClassLoader.getSystemResourceAsStream ("odbc.datasource");
      Properties p = new Properties();
      p.load (is);
      dataSource = p.getProperty("datasource.name");
      if (dataSource == null)
        throw new Exception ();
      username = p.getProperty("datasource.username", "");
      password = p.getProperty("datasource.password", "");
    } catch (Exception e) {
      System.out.println("Unable to read resource to get data source");
      return;
    }
    try { // Invalid connection
      Connection badcon = DriverManager.getConnection("foobar", username, password);
// If your datasource supports usernames and passwords, you can try this one too.
//      Connection con = DriverManager.getConnection(driverPrefixURL+dataSource, "DD", password);
    } catch (SQLException e) {
      System.out.println (e.getErrorCode() + ":" + e.getSQLState() + ":" + e.getMessage());
    }
    try { // Valid connection
      Connection con = null;
      try {
        con = DriverManager.getConnection(driverPrefixURL+dataSource+";foo=bar", username, password);
      } catch (SQLException e) {
        System.out.println ("foo=bar option failed");
        con = DriverManager.getConnection(driverPrefixURL+dataSource, username, password);
      }
      System.out.println("Connected.");
      // Check for connection warnings
      printSQLWarnings (con.getWarnings());

      DatabaseMetaData dmd = con.getMetaData();
      if (dmd == null) {
        System.out.println ("No Database Meta Data");
        return;
      } else {
        if (dmd.supportsANSI92FullSQL())
          System.out.println ("Supports Full ANSI92 SQL");
        else if (dmd.supportsANSI92IntermediateSQL()) 
          System.out.println ("Supports Intermediate ANSI92 SQL");
        else if (dmd.supportsANSI92EntryLevelSQL())
          System.out.println ("Supports Entry Level ANSI92 SQL");

        if (dmd.supportsExtendedSQLGrammar())
          System.out.println ("Supports Extended SQL Grammar");
        else if (dmd.supportsCoreSQLGrammar()) 
          System.out.println ("Supports Core SQL Grammar");
        else if (dmd.supportsMinimumSQLGrammar())
          System.out.println ("Supports Minimum SQL Grammar");
      }

      Statement stat = con.createStatement();
      printSQLWarnings (stat.getWarnings());

// Drop table students2 - so we can run this multiple times
// First time will display message, after that won't.

      try {
        stat.executeUpdate("drop table students2");
        printSQLWarnings (stat.getWarnings());
      } catch (SQLException e) {
        System.out.println ("Students2 table doesn't exist yet.");
      }
      stat.executeUpdate("create table students2 (name varchar (32), class varchar (32))");
      printSQLWarnings (stat.getWarnings());
      if ((dmd != null) && dmd.supportsCoreSQLGrammar()) {
        System.out.println ("Inserting the easy way");
        stat.executeUpdate ("insert into students2 select name, class from students");
        printSQLWarnings (stat.getWarnings());
      } else { // Minimum
        System.out.println ("Inserting the hard way");
        Statement stat2 = con.createStatement();
        ResultSet res = stat.executeQuery ("select name, class from students");
        printSQLWarnings (stat.getWarnings());
        String name, aClass;
        int nameCol = res.findColumn ("name");
        printSQLWarnings (res.getWarnings());
        int classCol = res.findColumn ("class");
        printSQLWarnings (res.getWarnings());
        int resultCount;
        while (res.next()) {
          name = res.getString (nameCol);
          printSQLWarnings (res.getWarnings());
          aClass = res.getString (classCol);
          printSQLWarnings (res.getWarnings());
          resultCount = stat2.executeUpdate (
            "insert into students2 (name, class) values ('" + 
            name + "', '" + aClass + "')");
          printSQLWarnings (stat2.getWarnings());
          if (resultCount != 1) {
            System.out.println ("Unable to insert: " + name + ":" + aClass);
          } else {
            System.out.println ("Inserted: " + name + ":" + aClass);
          }
        }
        stat2.close();
      }
      stat.close();
      con.close();
    } catch (SQLException e) {
      System.out.println (e.getErrorCode() + ":" + e.getSQLState() + ":" + e.getMessage());
      e.printStackTrace();
    }
  }
  private static void printSQLWarnings (SQLWarning w) {
    while (w != null) {
      System.out.println (w.getMessage());
      if (w instanceof DataTruncation) {
        DataTruncation d = (DataTruncation)w;
        System.out.println ("  Trunction error in column: " + d.getIndex());
      }
      w = w.getNextWarning();
    }
  }
}
