import java.awt.*;
import java.sql.*;
import java.util.Properties;
import java.io.InputStream;

public class Grades {
  private static NFBarchartApp bar;

  public static void main (String args[]) {
    String driverPrefixURL = "jdbc:odbc:";
    String username = null;
    String password = null;
    String dataSource = null;
    try {
      Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    } catch (Exception e) {
      System.out.println("Failed to load JDBC/ODBC driver.");
      return;
    }
    try {
      // Look for resource file 'odbc.datasource'
      InputStream is = ClassLoader.getSystemResourceAsStream ("odbc.datasource");
      Properties p = new Properties();
      p.load (is);
      dataSource = p.getProperty("datasource.name");
      if (dataSource == null)
        throw new Exception ();
      username = p.getProperty("datasource.username", "");
      password = p.getProperty("datasource.password", "");
    } catch (Exception e) {
      System.out.println("Unable to read resource to get data source");
      return;
    }
    try {
      Connection con = DriverManager.getConnection(driverPrefixURL+dataSource, username, password);
      Statement select = con.createStatement();
      ResultSet result = select.executeQuery("SELECT * FROM students order by name");

      int nameCol = result.findColumn ("name");
      int gradeCol = result.findColumn ("grade");
      String name, grade;
      StringBuffer names = new StringBuffer();
      StringBuffer averages = new StringBuffer();
      String previousName = null;
      float gradeSum = 0.0f;
      int classCount = 0;
      while(result.next()) {
        name = result.getString(nameCol);
        grade = result.getString(gradeCol);
        if (!name.equals (previousName)) {
          if (previousName != null) {
            float avg = gradeSum / classCount;
            if (names.length() != 0) names.append(",");
            names.append(previousName);
            if (averages.length() != 0) averages.append(",");
            averages.append(avg);
          }
          gradeSum = 0.0f;
          classCount = 0;
          previousName = name;
        }
        gradeSum += convertGrade (grade);
        classCount++;
      }
      if (previousName != null) {
        float avg = gradeSum / classCount;
        if (names.length() != 0) names.append(",");
        names.append(previousName);
        if (averages.length() != 0) averages.append(",");
        averages.append(avg);
      }
      select.close();
      con.close();
      // After closing connection
      if (names.length() != 0) {
        Frame f = new Frame ("Graph");
        showChartData (names.toString(), averages.toString());
        f.add(bar, BorderLayout.CENTER);
        f.setSize (300, 300);
        f.show();
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
  private static float convertGrade (String grade) {
    String trimmedGrade = grade.trim();
    float returnValue = 0;
    if (trimmedGrade.equals ("A"))
      returnValue = 4.0f;
    else if (trimmedGrade.equals ("A-"))
      returnValue = 3.667f;
    else if (trimmedGrade.equals ("B+"))
      returnValue = 3.333f;
    else if (trimmedGrade.equals ("B"))
      returnValue = 3.0f;
    else if (trimmedGrade.equals ("B-"))
      returnValue = 2.667f;
    else if (trimmedGrade.equals ("C+"))
      returnValue = 2.333f;
    else if (trimmedGrade.equals ("C"))
      returnValue = 2.0f;
    else if (trimmedGrade.equals ("C-"))
      returnValue = 1.667f;
    else if (trimmedGrade.equals ("D+"))
      returnValue = 1.333f;
    else if (trimmedGrade.equals ("D"))
      returnValue = 1.0f;
    else if (trimmedGrade.equals ("D-"))
      returnValue = 0.667f;
    return returnValue;
  }
  private static void showChartData (String names, String averages) {
    try {
      bar = new NFBarchartApp ();
      bar.init();
      bar.start();
      bar.loadParams ("Header      = ('Grade Report');" +
                      "HeaderBox   = (blue, SHADOW, 4);" +
                      "DataSets    = ('Grades', yellow);" +
                      "DataSet1    = "+ averages + ";" +
                      "BarLabels   = "+ names    + ";" +
                      "GraphLayout = VERTICAL;" +
                      "BottomAxis  = (green, 'TimesRoman', 14, 0, 0, 4)"
                     );
      bar.loadParams ("Update");
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}
